@@ -1,5 +1,5 @@
 # gophian -- tools to help with Debianizing Go software
-# Copyright (C) 2024 Maytham Alsudany <maytha8thedev@gmail.com>
+# Copyright (C) 2024-2025 Maytham Alsudany <maytha8thedev@gmail.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -37,6 +37,7 @@ from gophian.name import debianize_name, shorten_host
 from gophian.packages import DebianGolangPackages
 from gophian.salsa import salsa_repo_exists
 from gophian.session import Session
+from gophian.session.git import GoPackage
 from gophian.vcs import get_short_desc, package_from_import_path
 from gophian.wnpp import ITPBug, RFPBug
 
@@ -85,6 +86,12 @@ from gophian.wnpp import ITPBug, RFPBug
     help="Generate a debian/TODO file.",
     envvar="GOPHIAN_TODO",
 )
+@click.option(
+    "--pristine-tar/--no-pristine-tar",
+    default=False,
+    help="Create a pristine-tar branch and delta file.",
+    envvar="GOPHIAN_PRISTINE_TAR",
+)
 @click.option(
     "--git-remote",
     default="ssh",
@@ -94,6 +101,7 @@ from gophian.wnpp import ITPBug, RFPBug
 )
 @click.option(
     "--gpg-sign/--no-gpg-sign",
+    default=None,
     envvar="GOPHIAN_GPG_SIGN",
     help=(
         "GPG sign the commit that merges the upstream tarball into the"
@@ -101,6 +109,16 @@ from gophian.wnpp import ITPBug, RFPBug
         "to git nor gbp and they will use their configured defaults."
     ),
 )
+@click.option(
+    "--upstream-git-history/--no-upstream-git-history",
+    default=False,
+    envvar="GOPHIAN_UPSTREAM_GIT_HISTORY",
+    help=(
+        "Include upstream git history. Upstream will be tracked with the"
+        "'upstreamvcs' remote, and commits that import new package versions"
+        "will include the upstream commit or tag as a parent."
+    ),
+)
 @click.pass_context
 def make(
     context: click.Context,
@@ -113,8 +131,10 @@ def make(
     warn_salsa_repo: bool,
     itp: bool,
     todo: bool,
+    pristine_tar: bool,
     git_remote: str,
     gpg_sign: Optional[bool],
+    upstream_git_history: bool,
 ):
     """
     Prepare a Go module for packaging in Debian.
@@ -299,7 +319,7 @@ def make(
             keep_trailing_newline=True,
         )
 
-        version, repack, watchfile, tarball = session.prepare_upstream(
+        version, repack, watchfile, tarball, package = session.prepare_upstream(
             env,
             go_package,
             debian_source_package,
@@ -307,7 +327,7 @@ def make(
 
         tarball = move(tarball, work_dir.parent / tarball.name)
 
-        repo = Git.init(work_dir, "debian/sid", quiet=quiet)
+        repo = Git.init(work_dir, "debian/latest", quiet=quiet)
         if git_remote == "ssh":
             repo.remote(
                 f"git@salsa.debian.org:go-team/packages/{debian_source_package}.git"
@@ -317,18 +337,28 @@ def make(
                 f"https://salsa.debian.org/go-team/packages/{debian_source_package}.git"
             )
 
+        if upstream_git_history:
+            repo.remote(f"{package.repo}.git", "upstreamvcs")
+            repo.fetch("upstreamvcs")
+
         cmd = [
             "gbp",
             "import-orig",
             "--no-interactive",
-            "--debian-branch=debian/sid",
+            "--debian-branch=debian/latest",
+            "--upstream-branch=upstream/latest",
             "--filter=/debian/**/*",
-            tarball.absolute(),
+            "--pristine-tar" if pristine_tar else "--no-pristine-tar",
         ]
 
+        if upstream_git_history:
+            cmd.append(f"--upstream-vcs-tag={package.commitish}")
+
         if gpg_sign is not None:
             cmd.append("--sign-tags" if gpg_sign else "--no-sign-tags")
 
+        cmd.append(tarball.absolute())
+
         out = subprocess.run(
             cmd,
             stdout=subprocess.DEVNULL,
@@ -338,12 +368,6 @@ def make(
         if out.returncode != 0:
             raise GbpExecutionError(out)
 
-        with open(work_dir / ".gitignore", "a") as file:
-            file.write("\n/.pc/\n/_build/")
-
-        repo.add([".gitignore"])
-        repo.commit("Add quilt and build directories to gitignore", gpg_sign=gpg_sign)
-
         debian_deps = []
         debian_build_deps = [
             "debhelper-compat (= 13)",
@@ -501,6 +525,7 @@ def make(
                 "*.log\n"
                 "*.substvars\n"
                 "/.debhelper/\n"
+                "/build/\n"
                 "/debhelper-build-stamp/\n"
                 "/files\n"
             )
@@ -515,7 +540,7 @@ def make(
                 "#!/usr/bin/make -f\n"
                 "\n"
                 "%:\n"
-                "\tdh $@ --builddirectory=_build --buildsystem=golang\n"
+                "\tdh $@ --builddirectory=debian/build --buildsystem=golang\n"
             )
             if package_type == PackageType.BINARY:
                 file.write(
@@ -531,8 +556,14 @@ def make(
         with (work_dir / "debian/source/format").open("x") as file:
             file.write("3.0 (quilt)\n")
 
+        gbp_template = env.get_template("gbp.jinja")
+
         with (work_dir / "debian/gbp.conf").open("x") as file:
-            file.write("[DEFAULT]\n" "debian-branch = debian/sid\n" "dist = DEP14\n")
+            file.write(
+                gbp_template.render(
+                    pristine_tar=pristine_tar, upstream_git_history=upstream_git_history
+                )
+            )
 
         if (
             package_type == PackageType.BINARY_AND_LIBRARY
@@ -561,16 +592,10 @@ def make(
         with (work_dir / "debian/upstream/metadata").open("x") as file:
             file.write(gen_debian_upstream_metadata(git_repo))
 
-        with (work_dir / "debian/gitlab-ci.yml").open("x") as file:
-            file.write(
-                """# auto-generated, DO NOT MODIFY.
-# The authoritative copy of this file lives at:
-# https://salsa.debian.org/go-team/infra/pkg-go-tools/blob/master/config/gitlabciyml.go
----
-include:
-  - https://salsa.debian.org/go-team/infra/pkg-go-tools/-/raw/master/pipeline/test-archive.yml
-"""
-            )
+        salsa_ci_template = env.get_template("salsa-ci.jinja")
+
+        with (work_dir / "debian/salsa-ci.yml").open("x") as file:
+            file.write(salsa_ci_template.render())
 
         if itp:
             itp_template = env.get_template("itp.jinja")
@@ -837,18 +862,18 @@ class MakeSession(Session):
 
     def prepare_upstream(
         self, env: Environment, go_package: str, source_package: str
-    ) -> Tuple[str, bool, Path, Path]:
-        repo = self.go_get(go_package)
+    ) -> Tuple[str, bool, Path, Path, GoPackage]:
+        package = self.go_get(go_package)
         watch_path = self.tempdir.path / f"{source_package}_watch"
 
-        repack = (repo.path / "vendor").exists()
+        repack = (package.path / "vendor").exists()
 
-        version = repo.debianized_version
+        version = package.debianized_version
         if repack:
             version += "+ds"
 
         with watch_path.open("x") as file:
-            file.write(gen_debian_watch(env, repo.repo, repo.versioned, repack))
+            file.write(gen_debian_watch(env, package.repo, package.versioned, repack))
 
         uscan_args = [
             "uscan",
@@ -887,4 +912,4 @@ class MakeSession(Session):
 
         tarball_path = self.tempdir.path / Path(target_path)
 
-        return (version, repack, watch_path, tarball_path)
+        return (version, repack, watch_path, tarball_path, package)
